#!/usr/bin/env python
# coding: utf-8

# In[27]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[28]:


data30 = np.genfromtxt("Temp30.txt")
data40 = np.genfromtxt("Temp40.txt")
data50 = np.genfromtxt("Temp50.txt")
data60 = np.genfromtxt("Temp60.txt")


# In[29]:


time, exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60 = (
    data30[:, 0],
    data30[:, 1],
    data40[:, 1],
    data50[:, 1],
    data60[:, 1],
)


# In[30]:


exp_conc = np.array([exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60]).T


# In[31]:


temp = np.array([30.0, 40.0, 50.0, 60.0])
temp += 273.15


# In[32]:


k0, rorder, alfa = 0.1, 2, 0.05


# In[33]:


def obj_fun(x, i, rorder, time, exp_conc):
    k = x
    ddt = lambda t, c, k, rorder: -k * c ** rorder
    sol = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=[exp_conc[0, i]],
        t_eval=time,
        args=(k, rorder),
        method="LSODA",
    ).y.T
    return exp_conc[1:, i] - sol[1:, 0]


# In[34]:


k, ci, r2, r2adj = (
    np.empty(len(temp)),
    np.empty(len(temp)),
    np.empty(len(temp)),
    np.empty(len(temp)),
)
resid, jac = np.empty([len(time) - 1, len(temp)]), np.empty([len(time) - 1, len(temp)])


# In[38]:


for temp_idx, temp_value in enumerate(temp):
    param = least_squares(fun=obj_fun, 
                          x0=[k0], 
                          args=(temp_idx, rorder, time, exp_conc))
    k[temp_idx] = param.x
    resid[:, temp_idx] = param.fun
    jac[:, [temp_idx]] = param.jac  # .reshape(8,1)
    n, p = np.shape(jac[:, [temp_idx]])
    df = n - p
    ssr = np.sum(resid[:, temp_idx] ** 2)
    ssar = np.sum((np.mean(exp_conc[1:, temp_idx]) - exp_conc[1:, temp_idx]) ** 2)
    r2[temp_idx] = (1 - ssr / ssar) * 100
    r2adj[temp_idx] = 100 - (100 - r2[temp_idx]) * (n - 1) / (n - p - 1)
    s2 = ssr / df
    nu = inv(jac[:, [temp_idx]].T @ jac[:, [temp_idx]])
    ci[temp_idx] = np.multiply(
        t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu)))
    )


# In[39]:


for temp_idx, temp_value in enumerate(temp):
    print(
        "k(T={0:.0f} oC) = {1:.2E} +/- {2:.2E}\tError = {3:.2%}\tR2 = {4:.2f}%\tR2adj = {5:.2f}%"
        .format(temp_value - 273.15, k[temp_idx], ci[temp_idx], ci[temp_idx] / k[temp_idx], r2[temp_idx], r2adj[temp_idx])
    )


# In[40]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
c = np.empty([len(tspan), len(temp)])
ddt = lambda t, c, k, rorder: -k * c ** rorder
for temp_idx, temp_value in enumerate(temp):
    c[:, temp_idx] = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=[exp_conc[0, temp_idx]],
        t_eval=tspan,
        args=(k[temp_idx], rorder),
        method="LSODA",
    ).y


# In[ ]:


model = np.polyfit(1 / temp, np.log(k), 1)
ea = model[0] * (-RGAS)
pre_fac = np.exp(model[1])
print("A = {0:.2E} [check unit]\tEa = {1:.1f} kJ/mole".format(pre_fac, ea / 1000))


# In[ ]:


color_lab = "kbrg"


# In[ ]:


for temp_idx, temp_value in enumerate(temp):
    plt.plot(
        time,
        exp_conc[:, temp_idx],
        "o",
        color=color_lab[temp_idx],
        label="Exp. data (T= %.0f $^0$C)" % (temp_value - 273.15),
    )
    plt.plot(
        tspan,
        c[:, temp_idx],
        "-",
        color=color_lab[temp_idx],
        label="Est. data (T= %.0f $^0$C)" % (temp_value - 273.15),
    )

plt.title("Data fitting", fontsize=18)
plt.xlabel("time [s]", fontsize=16)
plt.ylabel("Concentration [mmol/L]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="temperatures",
    title_fontsize=16,
)
plt.show()


# In[ ]:


plt.plot(1 / temp, np.log(k), "o", label="Data")
plt.plot(1 / temp, np.polyval(model, 1 / temp), "-", label="Fit")
plt.title("Arrhenius Plot", fontsize=18)
plt.xlabel("1/T [K$^{-1}$]", fontsize=16)
plt.ylabel("ln(k$_{ref}$)", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    loc="upper right",
    fontsize=14,
)
plt.show()

