#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


data30 = np.genfromtxt("Temp30.txt")
data40 = np.genfromtxt("Temp40.txt")
data50 = np.genfromtxt("Temp50.txt")
data60 = np.genfromtxt("Temp60.txt")


# In[3]:


time, exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60 = (
    data30[:, 0],
    data30[:, 1],
    data40[:, 1],
    data50[:, 1],
    data60[:, 1],
)


# In[4]:


exp_conc = np.array([exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60]).T


# In[5]:


temp = np.array([30.0, 40.0, 50.0, 60.0])
temp += 273.15
temp_ref = 45
temp_ref += 273.15


# In[6]:


exp, alfa = 2, 0.05
kref0, ea0 = 0.01, 80
param0 = [kref0, ea0]


# In[7]:


def ddt(t, c, *args):
    kref, ea, temp, temp_ref, exp = args
    k = kref * np.exp(-ea * 1e3 / RGAS * (1 / temp - 1 / temp_ref))
    return -k * c ** exp


# In[8]:


def obj_fun(x, *args):
    exp, time, exp_conc, temp, temp_ref = args
    sol = np.empty([len(time), len(temp)])
    for temp_idx, temp_value in enumerate(temp):
        sol[:, [temp_idx]] = solve_ivp(
            fun=ddt,
            t_span=(time[0], time[-1]),
            y0=[exp_conc[0, temp_idx]],
            t_eval=time,
            args=(*x, temp_value, temp_ref, exp),
            method="LSODA",
        ).y.T
    return (exp_conc[1:, :] - sol[1:, :]).flatten()


# In[9]:


param = least_squares(
    fun=obj_fun, x0=param0, args=(exp, time, exp_conc, temp, temp_ref), method="lm"
)


# In[10]:


ci, resid, jac = (
    np.empty(len(param0)),
    np.empty([len(time), len(temp)]),
    np.empty([len(time), len(temp)]),
)


# In[11]:


kref, ea = param.x
resid = param.fun
jac = param.jac
n, p = np.shape(jac)
df = n - p
ssr = np.sum(resid ** 2)
ssar = np.sum((np.mean(exp_conc[1:, :]) - exp_conc[1:, :]) ** 2)
r2 = (1 - ssr / ssar) * 100
r2adj = 100 - (100 - r2) * (n - 1) / (n - p - 1)
s2 = ssr / df
nu = inv(jac.T @ jac)
ci = np.multiply(t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu))))


# In[12]:


print("R2 = {0:.2f}%\tR2adj = {1:.2f}%".format(r2, r2adj))
print(
    "kref(temp_ref={0:.0f} oC) = {1:.2E} +/- {2:.2E}\tError = {3:.2%}"
    .format(temp_ref - 273.15, kref, ci[0], ci[0] / kref)
)
print("Ea = {0:.1f} +/- {1:.1f} kJ/mole\tError = {2:.1%}".format(ea, ci[1], ci[1] / ea))


# In[13]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
c = np.empty([len(tspan), len(temp)])
for temp_idx, temp_value in enumerate(temp):
    c[:, [temp_idx]] = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=[exp_conc[0, temp_idx]],
        t_eval=tspan,
        args=(*param.x, temp_value, temp_ref, exp),
        method="LSODA",
    ).y.T


# In[14]:


color_lab = "kbrg"


# In[15]:


for temp_idx, temp_value in enumerate(temp):
    plt.plot(
        time,
        exp_conc[:, temp_idx],
        "o",
        color=color_lab[temp_idx],
        label="Exp. data (T= %.0f $^0$C)" % (temp_value - 273.15),
    )
    plt.plot(
        tspan,
        c[:, temp_idx],
        "-",
        color=color_lab[temp_idx],
        label="Est. data (T= %.0f $^0$C)" % (temp_value - 273.15),
    )

plt.title("Data fitting", fontsize=18)
plt.xlabel("time [s]", fontsize=16)
plt.ylabel("Concentration [mmol/L]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.ylim([0, None])
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="temperatures",
    title_fontsize=16,
)
plt.show()

