#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go
from plotly.subplots import make_subplots
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
import plotly.io as pio
pio.renderers.default = 'browser'

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


data30 = np.genfromtxt("Temp30.txt")
data40 = np.genfromtxt("Temp40.txt")
data50 = np.genfromtxt("Temp50.txt")
data60 = np.genfromtxt("Temp60.txt")


# In[3]:


time, exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60 = (
    data30[:, 0],
    data30[:, 1],
    data40[:, 1],
    data50[:, 1],
    data60[:, 1],
)


# In[4]:


exp_conc = np.array([exp_conc_30, exp_conc_40, exp_conc_50, exp_conc_60]).T


# In[5]:


temp = np.array([30.0, 40.0, 50.0, 60.0])
temp += 273.15
temp_ref = 45
temp_ref += 273.15


# In[6]:


exp, alfa = 2, 0.05
kref0, ea0 = 0.01, 80
param0 = [kref0, ea0]


# In[7]:


def ddt(t, c, *args):
    kref, ea, temp, temp_ref, exp = args
    k = kref * np.exp(-ea * 1e3 / RGAS * (1 / temp - 1 / temp_ref))
    return -k * c ** exp


# In[8]:


def obj_fun(x, *args):
    exp, time, exp_conc, temp, temp_ref = args
    sol = np.empty([len(time), len(temp)])
    for temp_idx, temp_value in enumerate(temp):
        sol[:, [temp_idx]] = solve_ivp(
            fun=ddt,
            t_span=(time[0], time[-1]),
            y0=[exp_conc[0, temp_idx]],
            t_eval=time,
            args=(*x, temp_value, temp_ref, exp),
            method="LSODA",
        ).y.T
    return (exp_conc[1:, :] - sol[1:, :]).flatten()


# In[9]:


param = least_squares(
    fun=obj_fun, x0=param0, args=(exp, time, exp_conc, temp, temp_ref), method="lm"
)
kref, ea = param.x


# In[10]:


surf_points = 60
ea_interv = np.linspace(ea / 3, ea * 3, surf_points)
k_interv = np.linspace(kref / 3, kref * 3, surf_points)
sensitivity = np.empty([surf_points, surf_points])


# In[11]:


for i in range(surf_points):
    for j in range(surf_points):
        sensitivity[i, j] = np.sum(
            (obj_fun([k_interv[j], ea_interv[i]], exp, time, exp_conc, temp, temp_ref))
            ** 2
        )


# In[12]:


fig = go.Figure(
    data=[go.Surface(x=ea_interv, y=k_interv, z=sensitivity, opacity=0.9)],
)

fig.update_traces(
    contours_z=dict(
        show=True,
        usecolormap=True,
        highlightcolor="limegreen",
        project_z=True,
    ),
    colorbar=dict(title="[mmole/L]<sup>2</sup>"),
)

fig.add_scatter3d(
    x=ea.flatten(),
    y=kref.flatten(),
    z=np.sum(param.fun ** 2).flatten(),
    mode="markers",
    #                   hovertemplate="y: %{y}",
    marker=dict(size=6, color="black"),
)

fig.update_layout(
    title="3D sensitivity Plot",
    autosize=True,
    width=1000,
    height=600,
    margin=dict(l=10, r=10, b=15, t=70),
    scene=dict(
        xaxis_title="ea [kJ/mole]",
        yaxis_title="k(T=45<sup>0</sup>C)",
        zaxis_title="[mmole/L]<sup>2</sup>",
    ),
)

fig.update_traces(
    hovertemplate="ea= %{x:.1f} kJ/mol<br>k(T=45<sup>0</sup>C)= %{y:.2e}<br> \
[mmole/L]<sup>2</sup>= %{z:.1f}<extra></extra>",
)
    
fig.show()

