#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


data120 = np.genfromtxt("120.txt")
data125 = np.genfromtxt("125.txt")
data130 = np.genfromtxt("130.txt")
data135 = np.genfromtxt("135.txt")
data142 = np.genfromtxt("142.txt")
data147 = np.genfromtxt("147.txt")
data150 = np.genfromtxt("150.txt")


# In[3]:


time = data120[:, 0] * 60


# In[4]:


exp_conc = np.concatenate(
    [
        data120[:, 1:],
        data125[:, 1:],
        data130[:, 1:],
        data135[:, 1:],
        data142[:, 1:],
        data147[:, 1:],
        data150[:, 1:],
    ],
    axis=1,
)


# In[5]:


_, num_comp = data120.shape
num_comp -= 1


# In[6]:


temp = np.array([120, 125, 130, 135, 142, 147, 150]) + 273.15
temp_ref = temp.mean()
alfa = 0.05


# In[7]:


init0 = [1e-6, 10, 1e-6, 10, 1e-6, 10, 1e-6, 10]


# In[8]:


def ddt(t, y, *args):
    x, temp, temp_ref, num_comp = args
    kref, ea = x[range(0, 8, 2)], x[range(1, 9, 2)]

    c_at, c_ac, c_pt, c_pc, c_st, c_sc = y

    k = kref * np.exp(-ea * 1e3 / RGAS * (1.0 / temp - 1.0 / temp_ref))

    result = np.empty(num_comp)

    result[0] = -(k[1] + k[3]) * c_at
    result[1] = -(k[0] + k[2]) * c_ac
    result[2] = k[1] * c_at
    result[3] = k[0] * c_ac
    result[4] = k[3] * c_at
    result[5] = k[2] * c_ac

    return result


# In[9]:


def obj_fun(x, *args):
    time, exp_conc, num_comp, temp, temp_ref = args
    sol = np.empty(exp_conc.shape)
    for temp_idx, temp_value in enumerate(temp):
        sol[:, temp_idx * num_comp : num_comp * (temp_idx + 1)] = solve_ivp(
            fun=ddt,
            t_span=[time[0], time[-1]],
            y0=exp_conc[0, temp_idx * num_comp : num_comp * (temp_idx + 1)],
            t_eval=time,
            args=(x, temp_value, temp_ref, num_comp),
        ).y.T

    return (exp_conc[1:, :] - sol[1:, :]).flatten()


# In[10]:


param = least_squares(
    fun=obj_fun,
    x0=init0,
    args=(time, exp_conc, num_comp, temp, temp_ref,),
    method="lm",
)


# In[11]:


kref, ea = param.x[range(0, 8, 2)], param.x[range(1, 9, 2)]


# In[12]:


ci, resid, jac = (
    np.empty(len(init0)),
    np.empty([len(time), len(temp)]),
    np.empty([len(time), len(temp)]),
)


# In[13]:


resid = param.fun
jac = param.jac
n, p = np.shape(jac)
df = n - p
ssr = np.sum(resid ** 2)
ssar = np.sum((np.mean(exp_conc[1:, :]) - exp_conc[1:, :]) ** 2)
r2 = (1 - ssr / ssar) * 100
r2adj = 100 - (100 - r2) * (n - 1) / (n - p - 1)
s2 = ssr / df
nu = inv(jac.T @ jac)
ci = np.multiply(t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu))))


# In[14]:


print("R2={0:.2f} %\tR2adj.={1:.2f} %%".format(r2, r2adj))
for kref_idx, kref_value in enumerate(kref):
    print(
        "k{0:d}(T={1:.0f} oC)={2:.2E} +/- {3:.2E}\tError={4:.1%}\tEa{5:d}={6:.1f} +/- {7:.1f} kJ/mole\tError={8:.1%}"
        .format(kref_idx + 1, 
                temp_ref - 273.15,
                kref_value,ci[2 * kref_idx],
                ci[2 * kref_idx] / kref_value,
                kref_idx + 1,
                ea[kref_idx],
                ci[2 * kref_idx + 1], 
                ci[2 * kref_idx + 1] / ea[kref_idx],
        )
    )


# In[15]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
c = np.empty([tint, len(exp_conc) * len(temp)])

for temp_idx, temp_value in enumerate(temp):
    c[:, temp_idx * num_comp : num_comp * (temp_idx + 1)] = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=exp_conc[0, temp_idx * num_comp : num_comp * (temp_idx + 1)],
        t_eval=np.linspace(time[0], time[-1], 300),
        args=(param.x, temp_value, temp_ref, num_comp),
    ).y.T


# In[16]:


lab_comp = ["AT", "AC", "PT", "PC", "ST", "SC"]
color_lab = "kbrgym"


# In[17]:


fig, axs = plt.subplots(len(temp))

for temp_idx, temp_value in enumerate(temp):
    for j in range(temp_idx * num_comp, num_comp * (temp_idx + 1)):
        axs[temp_idx].plot(time / 60, exp_conc[:, j], color_lab[j - temp_idx * num_comp] + "o")
        axs[temp_idx].plot(
            tspan / 60,
            c[:, j],
            color_lab[j - temp_idx * num_comp] + "-",
            label=lab_comp[j - temp_idx * num_comp],
        )
        axs[temp_idx].set_xlabel("time [min]", fontsize=16)
        axs[temp_idx].set_ylabel("Concentration [mole/kg]", fontsize=16)
        axs[temp_idx].set_title("Experiment at T=%.0f $^0$C" % (temp_value - 273.15), fontsize=18)
        axs[temp_idx].tick_params(axis="x", labelsize=14)
        axs[temp_idx].tick_params(axis="y", labelsize=14)
        axs[temp_idx].legend(
            bbox_to_anchor=(1.05, 1),
            loc="upper left",
            fontsize=14,
            title="Component",
            title_fontsize=16,
        )

plt.gcf().set_size_inches(10, 6 * len(temp))
fig.tight_layout()

