#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


data90 = np.genfromtxt("Exp90.txt")
data110 = np.genfromtxt("Exp110.txt")
data120 = np.genfromtxt("Exp120.txt")
data130 = np.genfromtxt("Exp130.txt")


# In[3]:


time = data90[:, 0] * 60
exp_conc = np.concatenate(
    [data90[:, 1:], data110[:, 1:], data120[:, 1:], data130[:, 1:]], axis=1
)


# In[4]:


temp = np.array([90, 110, 120, 130])
temp = temp + 273.15
temp_ref = np.mean(temp)
alfa = 0.05


# In[5]:


kp0, ea0, k10, ea10 = 1e-5, 50, 1e-5, 60
init0 = np.array([kp0, ea0, k10, ea10])


# In[6]:


def ddt(t, y, *args):
    (kp_ref, ea_p, k1_ref, ea1, temp, temp_ref,) = args

    c = y
    kp = kp_ref * np.exp(-ea_p * 1e3 / RGAS * (1.0 / temp - 1.0 / temp_ref))
    k1 = k1_ref * np.exp(-ea1 * 1e3 / RGAS * (1.0 / temp - 1.0 / temp_ref))

    return kp + k1 * c ** (0.5)


# In[7]:


def obj_fun(x, *args):
    (time, exp_conc, init0, temp, temp_ref,) = args
    sol = np.empty([len(time), len(temp)])
    for temp_idx, temp_value in enumerate(temp):
        sol[:, [temp_idx]] = solve_ivp(
            fun=ddt,
            t_span=(time[0], time[-1]),
            y0=[exp_conc[0, temp_idx]],
            t_eval=time,
            args=(*x, temp_value, temp_ref,),
        ).y.T

    return (exp_conc[1:, :] - sol[1:, :]).flatten()


# In[8]:


param = least_squares(
    fun=obj_fun,
    x0=init0.flatten(),
    args=(time, exp_conc, init0, temp, temp_ref,),
)


# In[9]:


kp_ref, ea_p, k1_ref, ea1 = param.x


# In[10]:


ci, resid, jac = (
    np.empty(len(init0)),
    np.empty([len(time), len(temp)]),
    np.empty([len(time), len(temp)]),
)
resid = param.fun
jac = param.jac
n, p = np.shape(jac)
df = n - p
ssr = np.sum(resid ** 2)
ssar = np.sum((np.mean(exp_conc[1:, :]) - exp_conc[1:, :]) ** 2)
r2 = (1 - ssr / ssar) * 100
r2adj = 100 - (100 - r2) * (n - 1) / (n - p - 1)
s2 = ssr / df
nu = inv(jac.T @ jac)
ci = np.multiply(t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu))))


# In[11]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
c = np.empty([len(tspan), len(temp)])
for temp_idx, temp_value in enumerate(temp):
    c[:, [temp_idx]] = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=[exp_conc[0, temp_idx]],
        t_eval=tspan,
        args=(*param.x, temp_value, temp_ref,),
    ).y.T


# In[12]:


print(
    "kp (T={0:.0f} oC)={1:.2E} +/- {2:.2E}\tError = {3:.1%} \t\tEa={4:.1f} +/- {5:.1f} kJ/mole\tError={6:.1%}"
    .format(temp_ref - 273.15, param.x[0], ci[0], ci[0] / kp_ref, ea_p,ci[1],ci[1] / ea_p,)
)


# In[13]:


color_lab = "kbrg"


# In[14]:


for i in range(len(temp)):
    plt.plot(
        time / 60,
        exp_conc[:, i],
        "o",
        color=color_lab[i],
        label="Exp. data (T= %.0f $^0$C)" % (temp[i] - 273.15),
    )
    plt.plot(
        tspan / 60,
        c[:, i],
        "-",
        color=color_lab[i],
        label="Est. data (T= %.0f $^0$C)" % (temp[i] - 273.15),
    )

plt.title("Data fitting", fontsize=18)
plt.xlabel("time [min]", fontsize=16)
plt.ylabel("MCA mole fraction [-]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Temperatures",
    title_fontsize=16,
)
plt.show()

