#!/usr/bin/env python
# coding: utf-8

# In[14]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# # ODE system

# In[15]:


def ddt(t, c, *args):

    (param, temp_ref, cp,rho, detal_hr, ca_in, reac_vol, vdot, temp0,) = args

    kref, ea = param
    c_a, c_b, temp = c

    cb_IN = 2 * ca_in
    c_in = np.array([ca_in, cb_IN])

    k = kref * np.exp(-ea * 1e3 / RGAS * (1 / temp - 1 / temp_ref))
    r = k * c_a * c_b
    SM = np.array([-2, -4])
    conc = np.array([c_a, c_b])

    rhs = np.empty(3)
    
    rhs[0:2] = vdot / reac_vol * (c_in - c[0:2]) + SM * r
    rhs[2] = vdot / reac_vol * (temp0 - temp) + r * (-detal_hr) / (rho * cp)

    return rhs


# In[16]:


def obj_fun(param, *args):
    (time, time0, temp_exp, init0, temp_ref, cp, rho, detal_hr, ca_in, reac_vol, vdot, temp0,) = args
    temp_est = solve_ivp(
        fun=ddt,
        t_span=(0., time[-1]),
        y0=init0,
        t_eval=time,
        args=(param, temp_ref, cp, rho, detal_hr, ca_in, reac_vol, vdot, temp0,),
    ).y[2, :]
    return temp_exp[1:] - temp_est[1:]


# # Loading and pre-processing of data

# In[17]:


data_exp = np.loadtxt("Data.txt")
time = data_exp[:, 0] * 60
time0 = data_exp[0, 0]

temp = data_exp[:, 1]
temp_exp = data_exp[:, 1] + 273.15

temp_ref = np.mean(temp_exp)

cp, rho, detal_hr, ca_in = 4.186e3, 1e3, -1004.3e3, 316.8
reac_vol, vdot, temp0 = 110e-6, 130e-6 / 60, 25.2 + 273.15

ca0, cb0 = 0.0, 0.0
temp0 = data_exp[0, 1] + 273.15
init0 = [ca0, cb0, temp0]

kref0, ea0 = 0.05, 80
alfa = 0.05


# # Processing of the data

# In[18]:


param = least_squares(
    fun=obj_fun,
    x0=[kref0, ea0],
    args=(time, time0, temp_exp, init0, temp_ref, cp, 
          rho, detal_hr, ca_in, reac_vol, vdot, temp0,),
)
kref, ea = param.x


# # Post-processing

# In[19]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
sol = solve_ivp(
    fun=ddt,
    t_span=(0, time[-1]),
    y0=init0,
    t_eval=tspan,
    args=(param.x, temp_ref, cp, rho, detal_hr, ca_in, reac_vol, vdot, temp0,),
)

temp_est = sol.y[2, :] - 273.15
c_est = sol.y[0:2, :]


# In[20]:


kref, ea = param.x
jac = param.jac
res = param.fun

n, p = np.shape(jac)
df = n - p

ssr = (res ** 2).sum()
ssar = ((temp[1:].mean() - temp[1:]) ** 2).sum()
s2 = ssr / df
nu = inv(jac.T @ jac)
ci = np.multiply(t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu))))
r2 = 100 * (1 - ssr / ssar)
r2adj = 100 - (100 - r2) * (n - 1) / (n - p - 1)


# In[21]:


print("R2=%.1f %%\tR2adj.=%.1f %%" % (r2, r2adj))
print(
    "k (T=%.0f oC) = %.2E +/- %.2E m3/mole/s\tError = %.1f %%\nEa = %.1f +/- %.1f kJ/mole\tError=%.1f %%"
    % (temp_ref - 273.15, kref, ci[0], ci[0] / kref * 100, ea, ci[1], ci[1] / ea * 100)
)


# # Visualizing the results

# In[22]:


plt.plot(time / 60, temp, "ko", label="Temperature experimental")
plt.plot(tspan / 60, temp_est, "k", label="Temperature fitted")
plt.title("Data fitting", fontsize=18)
plt.xlabel("time [min]", fontsize=16)
plt.ylabel("Temperature [$^o$C]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
plt.show()


# In[23]:


plt.plot(sol.t / 60, c_est[0, :], "b", label="Reactant A")
plt.plot(tspan / 60, c_est[1, :], "r", label="Reactanct B")
plt.title("Estimated Reactant concentrations", fontsize=18)
plt.xlabel("time [min]", fontsize=16)
plt.ylabel("concentration [mol/m$^3$]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    #           title='Initial temperature', title_fontsize=16,
)
plt.show()

