
import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 300


# # Data input

# In[2]:


temp_0 = 273.15 + 236.0
p_0 = 101.3e3
ndot_0 = 5.0 / 3600.0
x_a0 = 0.5
vdot_0 = ndot_0 * RGAS * temp_0 / p_0
ndot_a0 = p_0 * x_a0 * vdot_0 / (RGAS * temp_0)

dia = 0.05
area = np.pi * dia ** 2.0 / 4.0
length = 1.0
vr_total = area * length

k1 = 2.8e-3 / 60.0
k2 = 1.52e-4 / 60.0
SM = np.array([[-1, 1, 1, 0, 0], [-1, 0, 0, 1, 1]])

vr_span = np.linspace(0, vr_total, 200)
init_0 = np.array([ndot_a0, 0.0, 0.0, 0.0, 0.0])


# # ODE system

# In[3]:


def ddt(t, y, *args):

    ndot_a, ndot_b, ndot_c, ndot_d, ndot_e = y

    k1, k2, SM, temp0, p_0, ndot_0, vdot_0, ndot_a0 = args

    x1 = ndot_b / ndot_a0
    x2 = ndot_d / ndot_a0

    vdot = vdot_0 * (1.0 + x1 + x2)

    p_a = ndot_a * RGAS * temp_0 / vdot

    rate = np.empty(2)
    rate[0] = k1 * p_a
    rate[1] = k2 * p_a
    
    return SM.T @ rate


# # Processing of the data

# In[4]:


sol = solve_ivp(
    fun=ddt,
    t_span=(vr_span[0], vr_span[-1]),
    y0=init_0,
    t_eval=vr_span,
    method="LSODA",
    args=(k1, k2, SM, temp_0, p_0, ndot_0, vdot_0, ndot_a0),
).y.T


# # Post-processing data

# In[5]:


vdot = vdot_0 * (1.0 + sol[:, 1] / init_0[0] + sol[:, 3] / init_0[0])

length = vr_span / area
global_conv = ((init_0[0] - sol[:, 0]) / init_0[0]) * 100.0

for vr_idx, vr_value in enumerate(vr_span):
    if global_conv[vr_idx] > 95.0:
        vr_95 = vr_value
        vdot_95 = vdot[vr_idx]
        length_95 = length[vr_idx]
        break

tau_95 = vr_95 / vdot_95

print(
    "\n95% of formic acid conversion is achieved at the space-time of {0:.1f} s".format(
        tau_95
    )
)


# # Data visualization

# In[6]:


# %% Plotting molar flow vs. reactor length
fig, ax = plt.subplots(nrows=1, ncols=1)

ax.plot(length, sol * 3600)
ax.set_xlabel("Reactor length [m]", fontsize=16)
ax.set_ylabel("Molar Flow [mol/h]", fontsize=16)
ax.set_title("Molar Flow Profile", fontsize=18)
ax.tick_params(labelsize=14)
fig.set_size_inches(9, 6)
ax.legend(
    ["$CH_3OH$", "$H_2O$", "$CO$", "$H_2$", "$CO_2$"],
    loc="center right",
    fontsize=14,
    title="Concentrations",
    title_fontsize=16,
)
plt.show()


# In[7]:


# %% Plotting conversion vs. reactor length
fig, ax = plt.subplots(1, 1)
ax.plot(length, global_conv)
ax.plot([0, length_95, length_95], [95, 95, 0], "k--")
ax.set_xlabel("Reactor length [m]", fontsize=16)
ax.set_ylabel("Conversion [%]", fontsize=16)
ax.set_title("Conversion Profile", fontsize=18)
ax.tick_params(labelsize=14)
ax.legend(
    ["Conversion", "95% Conversion"],
    fontsize=14,
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
)
fig.set_size_inches(9, 6)
plt.show()


# In[10]:

# %% Plotting volumetric flow rate
fig, ax = plt.subplots(1, 1)
ax.plot(length, vdot * 3600)
ax.set_xlabel("Reactor length [m]", fontsize=16)
ax.set_ylabel("Volumetric Flow Rate $[m^3/s]$", fontsize=16)
ax.set_title("Volumetric Flow Rate Profile", fontsize=18)
ax.tick_params(labelsize=14)
fig.set_size_inches(9, 6)


# In[ ]:




