#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from numpy.linalg import inv
from scipy.integrate import solve_ivp
from scipy.optimize import least_squares
from scipy.stats import t

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


time = np.array([0, 426, 1150, 1660, 3120])
exp_conc = np.array([0.054, 0.0351, 0.0222, 0.0186, 0.0124])
conc0_triph = 0.1060


# In[3]:


def ddt(t, y, *args):
    k, conc0, conc0_triph = args
    c = y
    c_triph = conc0_triph - (conc0 - c)
    return -k * c ** 2 * c_triph ** 1


# In[4]:


def obj_fun(x, *args):
    time, exp_conc, conc0_triph = args
    k = x
    sol = solve_ivp(
        fun=ddt,
        t_span=(time[0], time[-1]),
        y0=[exp_conc[0]],
        t_eval=time,
        args=(k, exp_conc[0], conc0_triph),
        method="LSODA",
    ).y.reshape(len(time))

    return exp_conc[1:] - sol[1:]


# In[11]:


param = least_squares(
    fun=obj_fun, x0=0.01, args=(time, exp_conc, conc0_triph), method="lm"
)
k = param.x
jac = param.jac
res = param.fun

n, p = np.shape(jac)
df = n - p

ssr = np.sum(res ** 2)
ssar = np.sum((np.mean(exp_conc[1:]) - exp_conc[1:]) ** 2)
r2 = (1 - ssr / ssar) * 100
r2adj = 100 - (100 - r2) * (n - 1) / (n - p - 1)
s2 = ssr / df
alfa = 0.05
nu = inv(jac.T @ jac)
ci = np.multiply(t.ppf(1 - alfa / 2, df), np.sqrt(np.multiply(s2, np.diagonal(nu))))


# In[12]:


tint = 300
tspan = np.linspace(time[0], time[-1], tint)
c = solve_ivp(
    fun=ddt,
    t_span=(time[0], time[-1]),
    y0=[exp_conc[0]],
    t_eval=tspan,
    args=(k, exp_conc[0], conc0_triph),
    method="LSODA",
).y.flatten()


# In[13]:


print("Degree of explanation (r2) = {0:.2f} %%".format(r2))
print("Degree of explanation adjusted (r2adj) = {0:.2f}%".format(r2adj))
print("k = {0:.2e} +/- {1:.2e} \tEstimation error = {2:.2%}".format(k[0], ci[0], ci[0] / k[0]))


# In[14]:


plt.plot(time, exp_conc, "bo", label="Methanol - Experimental")
plt.plot(tspan, c, "r-", label="Methanol - Fitted")
plt.title("Data fitting", fontsize=18)
plt.xlabel("time [min]", fontsize=16)
plt.ylabel("Concentration [mole/L]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Concentration",
    title_fontsize=16,
)
plt.show()


# In[9]:


sens_interval = np.linspace(0, 6 * k, 100)
sensitivity = np.empty(len(sens_interval))
for sens_idx, sens_value in enumerate(sens_interval):
    sensitivity[sens_idx] = np.sum(obj_fun(sens_value, time, exp_conc, conc0_triph) ** 2)


# In[10]:


plt.plot(sens_interval, sensitivity, "b-")
plt.title("Sensitivity plot", fontsize=18)
plt.xlabel("k", fontsize=16)
plt.ylabel("Objective function [mole/L]$^2$", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.show()


# In[ ]:




