#!/usr/bin/env python
# coding: utf-8

# In[30]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import minimize

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[31]:


rho = 600e3
p0, ndot, mm = 101325, 20.67, 31.42
a, b, cp = -102.99e3, 8.33, 1.046
pre_fac1, ea1, pre_fac2, ea2 = 5.412e-3, 129791, 7.49e4, 224412


# In[32]:


lb, ub = 1e-6 * np.ones(3), 1 * np.ones(3)
temp0, y0, tau0 = (
    np.array([703, 703, 733]),
    np.array([0.085, 0.09, 0]),
    np.array([1, 1, 1]),
)
pa0_in, pb0_in, pc0_in = p0 * y0[0], p0 * y0[1], p0 * y0[2]


# In[33]:


def ddt(t, y, *args):

    (a, b, ndot, mm, temp0, p0, pa0, pb0, pc0, 
     pre_fac1, pre_fac2, ea1, ea2, rho, cp,) = args

    tau = t
    x, temp = y
    delta_hr = a + b * temp
    mdot = ndot * mm
    vdot0 = ndot * RGAS * temp0 / p0
    pa = pa0 * (1 - x)
    pb = pb0 - 1 / 2 * pa0 * x
    pc = pc0 + pa0 * x

    k1 = pre_fac1 * np.exp(-ea1 / RGAS / temp)
    k2 = pre_fac2 * np.exp(-ea2 / RGAS / temp)

    rate = k1 * pa * pb ** 0.5 - k2 * pc
    rA = -rate

    alfa = rho * vdot0 / temp0 * rate * (-delta_hr) / (mdot * cp)
    dtemp_dtau = alfa * temp / (1.0 - alfa * tau)

    return -rho * RGAS * rA / pa0 * (temp + tau * dtemp_dtau), dtemp_dtau


# In[34]:


temp0


# In[35]:


def obj_fun(t, *args):

    (a, b, ndot, mm, temp0, p0, pa0_in, pb0_in, pc0_in, 
     pre_fac1, pre_fac2, ea1, ea2, rho, cp,) = args

    pa0, pb0, pc0 = np.empty(4), np.empty(4), np.empty(4)
    pa0[0], pb0[0], pc0[0] = pa0_in, pb0_in, pc0_in
    conv = np.empty(3)

    for temp_idx, temp_value in enumerate(temp0):
        sol = solve_ivp(
            fun=ddt,
            t_span=(0, t[temp_idx]),
            y0=[0, temp0[temp_idx]],
            method="LSODA",
            args=(a, b, ndot, mm, temp_value, p0, pa0[temp_idx], pb0[temp_idx], pc0[temp_idx], 
                  pre_fac1, pre_fac2, ea1, ea2, rho,cp,),
        ).y.T

        conv[temp_idx] = sol[-1, 0]
        pa0[temp_idx + 1] = pa0[temp_idx] * (1 - conv[temp_idx])
        pb0[temp_idx + 1] = pb0[temp_idx] - 1 / 2 * pa0[temp_idx] * conv[temp_idx]
        pc0[temp_idx + 1] = pc0[temp_idx] + pa0[temp_idx] * conv[temp_idx]

    conv_globalA = (pa0[0] - pa0[-1]) / pa0[0] * 100
    treac = np.sum(t)
    if conv_globalA < 95:
        return treac * (95 - conv_globalA)
    else:
        return treac * (1 - conv_globalA / 100)


# In[36]:


t = minimize(
    fun=obj_fun,
    x0=[1, 1, 1],
    args=(a, b, ndot, mm, temp0, p0, pa0_in, pb0_in, pc0_in, 
          pre_fac1, pre_fac2, ea1, ea2, rho, cp,),
    method="Nelder-Mead",
    bounds=((1e-6, 5), (1e-6, 5), (1e-6, 5)),
)


# In[37]:


tint = 300
treac = np.array(
    [
        np.linspace(0, t.x[0], tint),
        np.linspace(0, t.x[1], tint),
        np.linspace(0, t.x[2], tint),
    ]
).T
conv_loc, conv_glob, temp = (
    np.empty([tint, 3]),
    np.empty([tint, 3]),
    np.empty([tint, 3]),
)
pa0, pb0, pc0 = np.empty(4), np.empty(4), np.empty(4)
pa0[0], pb0[0], pc0[0] = pa0_in, pb0_in, pc0_in

for temp_idx, temp_value in enumerate(temp0):
    sol = solve_ivp(
        fun=ddt,
        t_span=(0, treac[-1, temp_idx]),
        y0=[0, temp_value],
        method="LSODA",
        t_eval=treac[:, temp_idx],
        args=(a, b, ndot, mm, temp_value, p0, pa0[temp_idx], pb0[temp_idx], pc0[temp_idx], 
              pre_fac1, pre_fac2, ea1, ea2, rho, cp,),
    ).y.T

    conv_loc[:, temp_idx] = sol[:, 0]
    temp[:, temp_idx] = sol[:, 1]
    pa0[temp_idx + 1] = pa0[temp_idx] * (1 - conv_loc[-1, temp_idx])
    pb0[temp_idx + 1] = pb0[temp_idx] - 1 / 2 * pa0[temp_idx] * conv_loc[-1, temp_idx]
    pc0[temp_idx + 1] = pc0[temp_idx] + pa0[temp_idx] * conv_loc[-1, temp_idx]


# In[38]:


conv_glob[:, 0] = conv_loc[:, 0]
conv_glob[:, 1] = (pa0[0] - pa0[1] * (1.0 - conv_loc[:, 1])) / pa0[0]
conv_glob[:, 2] = (pa0[0] - pa0[2] * (1.0 - conv_loc[:, 2])) / pa0[0]


# In[46]:


for i in range(3):
    print("Reaction time reactor {0:d} = {1:.3f} s" .format(i, t.x[i]))
print("Total reaction time = {0:.3f} s".format(t.x.sum()))


# In[40]:


color_lab = "kbr"


# In[41]:


for i in range(3):
    plt.plot(
        treac[:, i],
        conv_loc[:, i] * 100,
        "-",
        color=color_lab[i],
        label="conversion reactor %i" % (i + 1),
    )

plt.title("Simulation", fontsize=18)
plt.xlabel("Time [s]", fontsize=16)
plt.ylabel("conversion [%]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="conversion profiles",
    title_fontsize=16,
)
plt.show()


# In[42]:


for i in range(3):
    plt.plot(
        treac[:, i],
        temp[:, i],
        "-",
        color=color_lab[i],
        label="Temperature reactor %i" % (i + 1),
    )

plt.title("Simulation", fontsize=18)
plt.xlabel("Time [s]", fontsize=16)
plt.ylabel("Temperature [K]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Temp. profiles",
    title_fontsize=16,
)
plt.show()


# In[43]:


for i in range(3):
    plt.plot(
        np.sum(treac[-1, 0:i]) + treac[:, i],
        conv_glob[:, i] * 100,
        "-",
        color=color_lab[i],
        label="Global conv. reactor %i" % (i + 1),
    )

plt.plot(
    np.array([0, t.x.sum(), t.x.sum()]),
    np.array([95, 95, 0]),
    "g--",
    label="95% conversion",
)

plt.title("Simulation", fontsize=18)
plt.xlabel("Time [s]", fontsize=16)
plt.ylabel("Global conversion [%]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.ylim([None, 100])
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Global conv. profiles",
    title_fontsize=16,
)
plt.show()

