#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[2]:


ea, pre_fac, delta_hr = 105e3, 1.37e9, -33.5e3
rho_cp, temp_0, vl_0 = 1980e3, 53 + 273.15, 5


# In[3]:


c_a0, c_b0, c_c0 = 4.55e3, 5.34e3, 0.0
SM = np.array([-1, -1, 1])
u_coeff = 250.0


# In[4]:


Init0 = [c_a0, c_b0, c_c0, temp_0]
t_max = 3 * 3600.0
points = 1000
area = np.linspace(0, 50, points)
time_span = np.linspace(0, t_max, 3600)


# In[5]:


def ddt(t, y, *args,):

    area, ea, pre_fac, delta_hr, rho_cp, temp_0, vl_0, SM, u_coeff = args

    cA, cB, cC, temp = y

    k = pre_fac * np.exp(-ea / RGAS / temp)
    r = k * cA * cB

    result = np.empty(4)
    result[0:3] = SM * r
    result[3] = (u_coeff * area * (temp_0 - temp) + r * (-delta_hr) * vl_0) / (
        rho_cp * vl_0
    )

    return result


# In[6]:


for area_value in area:
    sol = solve_ivp(
        fun=ddt,
        t_span=(time_span[0], time_span[-1]),
        y0=Init0,
        args=(area_value, ea, pre_fac, delta_hr,
              rho_cp, temp_0, vl_0, SM, u_coeff,),
        t_eval=time_span,
        method="LSODA",
    ).y.T

    temp_max = np.max(sol[:, -1]) - 273.15
    if temp_max < 100.0:
        conv = (c_a0 - sol[:, 0]) / c_a0 * 100.0
        if np.max(conv) > 95.0:
            for time_idx, time_value in enumerate(time_span):
                if conv[time_idx] >= 95.0:
                    t_min = time_value
                    areamin = area_value
                    break
            break


# In[7]:


sol = solve_ivp(
    fun=ddt,
    t_span=(time_span[0], time_span[-1]),
    y0=Init0,
    args=(areamin, ea, pre_fac, delta_hr, rho_cp, temp_0, vl_0, SM, u_coeff),
    t_eval=time_span,
    method="LSODA",
)


# In[8]:


t, conc, temp = sol.t.T / 60, sol.y.T[:, :3], sol.y.T[:, -1] - 273.15


# In[9]:


fig, ax = plt.subplots(1, 1)
ax.plot(t, conc[:, 0] / 1000, label="Maleic acid")
ax.plot(t, conc[:, 1] / 1000, label="Hexanol")
ax.plot(t, conc[:, 2] / 1000, label="Maleic acid monoester")
ax.set_xlim([0, t[-1]])
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Concentration [mol/L]", fontsize=16)
ax.set_title("Simulation", fontsize=18)
ax.tick_params(labelsize=14)
fig.set_size_inches(9, 6)
ax.legend(
    title="Concentrations",
    title_fontsize=16,
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
plt.show()


# In[10]:


fig, ax = plt.subplots(1, 1)
ax.plot([0, time_span[-1]], [100, 100], "k--", label="Max. Temperature")
ax.plot(t, temp, label="Temperature")
ax.set_xlim([0, t[-1]])
ax.set_ylim([np.min(temp), 105])
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Temperature [$^o$C]", fontsize=16)
ax.set_title("Simulation", fontsize=18)
ax.tick_params(labelsize=14)
plt.gcf().set_size_inches(9, 6)
ax.legend(loc="center right", fontsize=14)
plt.show()


# In[11]:


a_conv = (c_a0 - conc[:, 0]) / c_a0 * 100


# In[12]:


fig, ax = plt.subplots(1, 1)
ax.plot([0, t_min / 60, t_min / 60], [95, 95, 0], "k--")
ax.plot(t, a_conv)
ax.set_xlim([0, t[-1]])
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Conversion [%]", fontsize=16)
ax.set_title("Conversion [%] vs. Time [min]", fontsize=18)
ax.tick_params(labelsize=14)
fig.set_size_inches(9, 6)
plt.show()

