#!/usr/bin/env python
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import gas_constant as RGAS
from scipy.integrate import solve_ivp
from scipy.optimize import fsolve

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 300


# In[2]:


ea, pre_fac, delta_hr = 99e3, 1.52e9, -33.5e3
rho_cp, vdot, temp_0, reac_vol = 1900e3, 0.6e-3, 290, 1
c_a_in, c_b_in, c_c_in = 5e3, 5e3, 0
c_in = [c_a_in, c_b_in, c_c_in]
SM = np.array([[-1, -1, 1]])
tf = 200.0
tf *= 60
tspan = np.linspace(0, tf, 2000)
c_a_0, c_b_0, c_p_0 = 0.0, 0.0, 0.0
temp_init = range(290, 410, 10)


# In[3]:


def ddt(t, y, *args):

    c_a, c_b, c_p, temp = y

    ea, pre_fac, delta_hr, SM, c_in, rho_cp, vdot, temp_0, RGAS, reac_vol = args

    k = pre_fac * np.exp(-ea / RGAS / temp)
    rate = k * c_a * c_b

    result = np.empty([4])

    result[0:3] = vdot / reac_vol * (c_in - y[0:3]) + SM * rate
    result[3] = vdot / reac_vol * (temp_0 - temp) - rate * delta_hr / rho_cp

    return result


# # Processing

# In[6]:


sol = {}

for i, _ in enumerate(temp_init):
    Init0 = [c_a_0, c_b_0, c_p_0, temp_init[i]]
    sol[i] = solve_ivp(
                    fun=ddt,
                    t_span=(tspan[0], tspan[-1]),
                    y0=Init0,
                    t_eval=tspan,
                    method="LSODA",
                    args=(ea, pre_fac, delta_hr, SM, c_in, 
                          rho_cp, vdot, temp_0, RGAS, reac_vol),
                ).y.T


# # Data visualization below

# In[8]:


fig, ax = plt.subplots(1, 1)
for i, _ in enumerate(temp_init):
    color = np.linspace(0, 1.0, len(temp_init))
    ax.plot(
        tspan / 60,
        sol[i][:, 0],
        label="T = " + "{0:.0f}".format(temp_init[i]) + " $^o$C",
        color=(color[i], 0, color[i]),
    )
    ax.plot(tspan / 60, sol[i][:, 0], color=(color[i], 0, color[i]))

ax.legend(
    title="Initial temperature",
    title_fontsize=16,
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Concentration [mole/m$^3$]", fontsize=16)
ax.set_title("Reactant concentration (A&B) [mole/m$^3$]", fontsize=18)
ax.tick_params(labelsize=14)
ax.set_xlim([0.0, None])
ax.set_ylim([0.0, None])
fig.set_size_inches(10, 6)
plt.show()


# In[10]:


fig, ax = plt.subplots(1, 1)
for i, _ in enumerate(temp_init):
    color = np.linspace(0, 1.0, len(temp_init))
    ax.plot(
        tspan / 60,
        sol[i][:,2],
        label="T = " + "{0:.0f}".format(temp_init[i]) + " $^o$C",
        color=(color[i], 0, color[i]),
    )

ax.legend(
    title="Initial temperature",
    title_fontsize=16,
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Concentration [mole/m$^3$]", fontsize=16)
ax.set_title("Product concentration (C) [mole/m$^3$]", fontsize=18)
ax.tick_params(labelsize=14)
ax.set_xlim([0, None])
ax.set_ylim([0, None])
fig.set_size_inches(10, 6)
plt.show()


# In[12]:


fig, ax = plt.subplots(1, 1)
for i, _ in enumerate(temp_init):
    color = np.linspace(0, 1.0, len(temp_init))
    ax.plot(
        tspan / 60,
        sol[i][:, 3],
        label="T = " + "{0:.0f}".format(temp_init[i]) + " $^o$C",
        color=(color[i], 0, color[i]),
    )

ax.legend(
    title="Initial temperature",
    title_fontsize=16,
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
ax.set_xlabel("Time [min]", fontsize=16)
ax.set_ylabel("Temperature [$^o$C]", fontsize=16)
ax.set_title("Reactor temperature [$^o$C]", fontsize=18)
ax.tick_params(labelsize=14)
ax.set_xlim([0, None])
fig.set_size_inches(10, 6)
plt.show()


# In[13]:


def temp_steady(temp, temp_0, tau, ea, pre_fac, delta_hr, rho_cp, c_a_0, RGAS):

    alfa = pre_fac * np.exp(-ea / RGAS / temp) * tau
    c_a = (-1.0 + np.sqrt(1.0 + 4.0 * alfa * c_a_0)) / (2.0 * alfa)

    return (temp_0 - temp) / tau - pre_fac * np.exp(
        -ea / RGAS / temp
    ) * c_a ** 2.0 * delta_hr / rho_cp


# In[14]:


temp_domain = np.linspace(290, 400, 200)
temp_zero = np.empty(len(temp_domain))
tau = reac_vol / vdot

for i, _ in enumerate(temp_domain):
    temp_zero[i] = temp_steady(
        temp_domain[i], temp_0, tau, ea, pre_fac, delta_hr, rho_cp, c_a_in, RGAS
    )


# In[15]:


fig, ax = plt.subplots(1, 1)
ax.plot(temp_domain - 273.14, temp_zero, "b")
ax.plot(temp_domain - 273.14, np.zeros(len(temp_domain)), "k--")
ax.set_title("Steady-State Temperature zero function", fontsize=18)
ax.set_xlabel("Temperature [K]", fontsize=16)
ax.set_ylabel("Fzero(SteadyStateTemp)", fontsize=16)
ax.set_xlim([temp_domain[0] - 273.14, temp_domain[-1] - 273.14])
fig.set_size_inches(10, 6)
ax.tick_params(labelsize=14)
plt.show()


# In[16]:


temp_guess = [290, 320, 370]
temp_steady_values = np.empty(3)
for i in range(3):
    temp_steady_values[i] = fsolve(
        func=temp_steady,
        x0=temp_guess[i],
        args=(temp_0, tau, ea, pre_fac, delta_hr, rho_cp, c_a_in, RGAS),
    )


# In[17]:


for i, j in enumerate(temp_steady_values):
    print("Steady-state solution {0} = {1:.1f} oC".format(i + 1, j - 273.15))

