#!/usr/bin/env python
# coding: utf-8

# In[30]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import R as RGAS
from scipy.integrate import simps, solve_ivp
from scipy.optimize import fsolve

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[31]:


pre_fac, ea = 94.3, 95.02
rho_p, eps_p, tau_p, rc, s = 1.55, 0.55, 2, 3.2, 2
temp = np.array([663, 683, 703, 723])
press = 1
y0 = np.array([0.07, 0.5, 0.03, 0.2, 0.2])
diff_kn_CO, temp_ref = 1.07e-4, 723
param_1, param_2 = 4577.8, 4.33

v = np.array([18, 13.1, 26.9, 6.12, 18.5])
molar_mass = np.array([28, 18, 44, 2, 28])

#%% Pre-processing of input data
rho_p *= 1e3
rc *= 1e-3
press *= 101325
diff_kn_CO *= 1e-4

#%%
num_comp = len(y0)
num_temp = len(temp)
SM = np.array([[-1, -1, 1, 1, 0]])
y0_center = np.ones((1, 5)) / 5


# In[32]:


def diff_calc(temp, *args):

    press, y, diff_kn_CO, eps_p, tau_p, v, molar_mass, temp_ref, num_comp = args

    diff_bin = np.zeros([num_comp, num_comp])
    diff_mi = np.empty(num_comp)
    diff_kni = np.empty(num_comp)
    diff_ei = np.empty(num_comp)

    for i, _ in enumerate(y):
        for j, _ in enumerate(y):
            if i != j:
                diff_bin[i, j] = (
                    temp ** 1.75
                    * (1.0 / molar_mass[i] + 1.0 / molar_mass[j]) ** 0.5
                    * 1e-7
                    / (press / 101325 * (v[i] ** (1 / 3) + v[j] ** (1 / 3)) ** 2.0)
                )

    for i, _ in enumerate(y):
        sum = 0.0
        for j, _ in enumerate(y):
            if i != j:
                sum += y[j] / diff_bin[i, j]

        diff_mi[i] = (1.0 - y[i]) / sum

    for i, _ in enumerate(y):
        diff_kn_aux1 = diff_kn_CO * (molar_mass[0] / molar_mass[i]) ** 0.5
        diff_kni[i] = diff_kn_aux1 * (temp / temp_ref) ** 0.5

    diff_ei = eps_p / tau_p * (1.0 / diff_mi + 1.0 / diff_kni) ** (-1.0)

    return diff_ei


# In[33]:


def theor_effec_fact(rc, *args):

    (rho_p, temp, press, y, diff_kn_CO, eps_p, tau_p,
     v, molar_mass, temp_ref, num_comp, y0, pre_fac, ea,) = args

    y_CO, y_H2O, y_CO2, y_H2, y_N2 = y

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)

    phi = (rc ** 2* k* rho_p
        / diff_calc(temp, press, y, diff_kn_CO, eps_p, 
                    tau_p, v, molar_mass, temp_ref, num_comp)[0]) ** 0.5

    return 3.0 / phi * (1.0 / np.tanh(phi) - 1.0 / phi)


# In[34]:


def effec_fact(y, *args):

    rc, s, y0, xspan, pre_fac, ea, temp, param_1, param_2, num_comp = args

    y_CO, y_H2O, y_CO2, y_H2, y_N2 = y[:, 0], y[:, 1], y[:, 2], y[:, 3], y[:, 4]

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)

    K = np.exp(param_1 / temp - param_2)

    r = k * y_CO * (1.0 - y_CO2 * y_H2 / (y_CO * y_H2O) / K)

    NumInteg = simps((xspan * rc) ** s * r, xspan)

    return (s + 1.0) * rc * NumInteg / (r[-1] * rc ** (s + 1.0))


# In[35]:


def ddx(t, y, *args):

    (pre_fac, ea, temp, param_1, param_2, num_comp, SM, rho_p, rc,
     diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref,) = args

    x = t
    y_CO, y_H2O, y_CO2, y_H2, y_N2 = y[0:num_comp]
    hi = y[num_comp:]

    diff_ei = diff_calc(temp,press, y[0:num_comp], diff_kn_CO,
                        eps_p, tau_p, v, molar_mass, temp_ref, num_comp,)

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)
    K = np.exp(param_1 / temp - param_2)
    r = k * y_CO * (1 - y_CO2 * y_H2 / (y_CO * y_H2O) / K)

    result = np.empty(2 * num_comp)

    result[0:num_comp] = hi
    result[num_comp:] = -2.0 / x * hi - SM * r * rho_p * rc ** 2 / diff_ei

    return result


# In[36]:


def shoot_fun(y_center, *args):

    (y0, pre_fac, ea, temp, param_1, param_2, num_comp, SM,
     rho_p, rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref,) = args

    sol = solve_ivp(
        fun=ddx,
        t_span=(1E-6, 1.),
        y0=np.append(y_center, np.zeros(num_comp)),
        method="BDF",
        args=(pre_fac, ea, temp, param_1, param_2, num_comp, SM, rho_p,
              rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref,),
    ).y.T

    return y0 - sol[-1, 0:num_comp]


# # Processsing

# In[37]:


sol = {}

for temp_idx, temp_value in enumerate(temp):

    y_center = fsolve(
        func=shoot_fun,
        x0=y0_center,
        args=(y0, pre_fac, ea, temp_value, param_1, param_2, num_comp,
              SM, rho_p, rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref,),
    )

    sol[temp_idx] = solve_ivp(
        fun=ddx,
        t_span=(1e-6, 1),
        y0=np.append(y_center, np.zeros(num_comp)),
        method="BDF",
        args=(pre_fac, ea,temp_value, param_1, param_2, num_comp, SM,
              rho_p, rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref,),
    )


# In[38]:


for temp_idx, temp_value in enumerate(temp):

    print(
        "Theoretical effectiveness factor (T={1:.0f} oC) = {0:.3f} [-]".format(
            theor_effec_fact(rc, rho_p, temp_value, press, y0, diff_kn_CO,
                             eps_p, tau_p, v, molar_mass, temp_ref, num_comp, y0, pre_fac, ea,),
            temp_value - 273.15,
        )
    )

    print(
        "Calculated effectiveness factor (T={1:.0f} oC) = {0:.3f} [-]\n".format(
            effec_fact(sol[temp_idx].y.T[:, 0:num_comp], rc, s, y0,sol[temp_idx].t,
                       pre_fac, ea,temp_value, param_1, param_2, num_comp,),
            temp_value - 273.15,
        )
    )


# In[39]:


labComp = ["CO", "H$_2$O", "CO$_2$", "H$_2$", "N$_2$"]
ColorLab = ["k", "b", "r", "g", "m"]

#%%
for temp_idx, temp_val in enumerate(temp):
    fig, axs = plt.subplots()

    for y0_idx, _ in enumerate(y0):
        axs.plot(
            sol[temp_idx].t,
            sol[temp_idx].y.T[:, y0_idx],
            ColorLab[y0_idx] + "-",
            label=labComp[y0_idx],
        )

    axs.set_xlabel("Dimensionless radius [-]", fontsize=16)
    axs.set_ylabel("Concentration [mole/m$^3$]", fontsize=16)
    axs.tick_params(axis="x", labelsize=14)
    axs.tick_params(axis="y", labelsize=14)
    axs.legend(
        bbox_to_anchor=(1.05, 1),
        loc="upper left",
        fontsize=14,
        title="Component",
        title_fontsize=16,
    )
    axs.set_title(f'Temperature {temp_val} $^o$C')
    plt.xlim([0.0, 1.0])
    plt.ylim([0.0, 0.52])
    plt.gcf().set_size_inches(10, 6)
    plt.show()

