#!/usr/bin/env python
# coding: utf-8

# In[12]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.constants import R as RGAS
from scipy.integrate import simps
from scipy.optimize import fsolve

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[13]:


pre_fac, ea = 94.3, 95.02
rho_p, eps_p, tau_p, rc, s = 1.55, 0.55, 2, 3.2, 2
temp = np.array([663, 683, 703, 723])
press = 1
y0 = np.array([0.07, 0.5, 0.03, 0.2, 0.2])
diff_kn_CO, temp_ref = 1.07e-4, 723
param_1, param_2 = 4577.8, 4.33

v = np.array([18, 13.1, 26.9, 6.12, 18.5])
molar_mass = np.array([28, 18, 44, 2, 28])

#%% Pre-processing of input data
rho_p *= 1e3
rc *= 1e-3
press *= 101325
diff_kn_CO *= 1e-4

#%%
num_comp = len(y0)
num_temp = len(temp)
n = 60
xspan = np.linspace(0, 1.0, n)
SM = np.array([[-1, -1, 1, 1, 0]])


# In[14]:


def diff_calc(temp, *args):

    press, y, diff_kn_CO, eps_p, tau_p, v, molar_mass, temp_ref, num_comp = args

    diff_bin = np.zeros([num_comp, num_comp])
    diff_mi = np.empty(num_comp)
    diff_kni = np.empty(num_comp)
    diff_ei = np.empty(num_comp)

    for i, _ in enumerate(y):
        for j, _ in enumerate(y):
            if i != j:
                diff_bin[i, j] = (
                    temp ** 1.75
                    * (1.0 / molar_mass[i] + 1.0 / molar_mass[j]) ** 0.5
                    * 1e-7
                    / (press / 101325 * (v[i] ** (1 / 3) + v[j] ** (1 / 3)) ** 2.0)
                )

    for i, _ in enumerate(y):
        sum = 0
        for j, _ in enumerate(y):
            if i != j:
                sum += y[j] / diff_bin[i, j]

        diff_mi[i] = (1.0 - y[i]) / sum

    for i, _ in enumerate(y):
        diff_kn_aux1 = diff_kn_CO * (molar_mass[0] / molar_mass[i]) ** 0.5
        diff_kni[i] = diff_kn_aux1 * (temp / temp_ref) ** 0.5

    diff_ei = eps_p / tau_p * (1.0 / diff_mi + 1.0 / diff_kni) ** (-1.0)

    return diff_ei


# In[15]:


def theor_effec_fact(rc, *args):

    (rho_p, temp, press, y, diff_kn_CO, eps_p, tau_p,
     v, molar_mass, temp_ref, num_comp, y0, pre_fac, ea,) = args

    y_CO, y_H2O, y_CO2, y_H2, y_N2 = y

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)

    phi = (rc ** 2* k* rho_p
        / diff_calc(temp, press, y, diff_kn_CO, eps_p, 
                    tau_p, v, molar_mass, temp_ref, num_comp)[0]) ** 0.5

    return 3.0 / phi * (1.0 / np.tanh(phi) - 1.0 / phi)


# In[16]:


def effec_fact(y, *args):

    rc, s, y0, xspan, pre_fac, ea, temp, param_1, param_2, num_comp = args

    y_CO, y_H2O, y_CO2, y_H2, y_N2 = y[:, 0], y[:, 1], y[:, 2], y[:, 3], y[:, 4]

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)

    K = np.exp(param_1 / temp - param_2)

    r = k * y_CO * (1.0 - y_CO2 * y_H2 / (y_CO * y_H2O) / K)

    NumInteg = simps((xspan * rc) ** s * r, xspan)

    return (s + 1.0) * rc * NumInteg / (r[-1] * rc ** (s + 1.0))


# In[17]:


def obj_fun(y, *args):

    (pre_fac, ea, temp, param_1, param_2, num_comp, SM, xspan, rho_p,
     rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref, temp,) = args

    y = y.reshape(num_comp, n - 2)

    y = np.concatenate(
        (
            ((4 * y[:, 0] - y[:, 1]) / 3).reshape(num_comp, 1),
            y,
            y0.reshape(num_comp, 1),
        ),
        1,
    )

    k = pre_fac * np.exp(-ea * 1e3 / RGAS / temp)
    K = np.exp(param_1 / temp - param_2)

    result = np.empty((num_comp, n))

    for i in range(1, n - 1):

        diff_ei = diff_calc(temp, press,y[:, i], diff_kn_CO, eps_p, tau_p,
                            v, molar_mass, temp_ref, num_comp,).reshape(num_comp, 1)

        r = k * y[0, i] * (1.0 - y[2, i] * y[3, i] / (y[0, i] * y[1, i]) / K)

        result[:, i] = (
            (y[:, i - 1] - 2 * y[:, i] + y[:, i + 1]) / (xspan[1] ** 2)
            + 2.0 / xspan[i] * (y[:, i + 1] - y[:, i - 1]) / (2 * xspan[1])
            + (SM.T * r * rho_p * rc ** 2 / diff_ei).flatten()
        )

    return result[:, 1:-1].flatten()


# # Processsing

# In[18]:


init0 = np.ones((num_comp, n - 2)) * y0.reshape(num_comp, 1) / 2

sol = {}
y = {}

for temp_idx, temp_value in enumerate(temp):

    sol[temp_idx] = fsolve(
        func=obj_fun,
        x0=init0,
        args=(pre_fac, ea, temp, param_1, param_2, num_comp, SM, xspan, rho_p,
              rc, diff_kn_CO, v, molar_mass, eps_p, tau_p, temp_ref, temp_value,),
        xtol=1e-10,
    )

    y[temp_idx] = sol[temp_idx].reshape(num_comp, n - 2)
    y[temp_idx] = np.concatenate(
        (
            ((4 * y[temp_idx][:, 0] - y[temp_idx][:, 1]) / 3).reshape(num_comp, 1),
            y[temp_idx],
            y0.reshape(num_comp, 1),
        ),
        1,
    ).T


# In[19]:


for temp_idx, temp_value in enumerate(temp):

    print(
        "Theoretical effectiveness factor (T={1:.0f} oC) = {0:.3f} [-]".format(
            theor_effec_fact(rc, rho_p, temp_value, press, y0, diff_kn_CO,
                             eps_p, tau_p, v, molar_mass, temp_ref, num_comp, y0, pre_fac, ea,),
            temp_value - 273.15,
        )
    )

    print(
        "Calculated effectiveness factor (T={1:.0f} oC) = {0:.3f} [-]\n".format(
            effec_fact(y[temp_idx], rc, s, y0, xspan, pre_fac,
                       ea, temp_value, param_1, param_2, num_comp,),
            temp_value - 273.15,
        )
    )


# In[20]:


labComp = ["CO", "H$_2$O", "CO$_2$", "H$_2$", "N$_2$"]
ColorLab = ["k", "b", "r", "g", "m"]

#%%
for i, temp_val in enumerate(temp):
    fig, axs = plt.subplots()

    for j, _ in enumerate(y0):
        axs.plot(xspan, y[i][:, j], ColorLab[j] + "-", label=labComp[j])

    axs.set_xlabel("Dimensionless radius [-]", fontsize=16)
    axs.set_ylabel("Concentration [mole/m$^3$]", fontsize=16)
    axs.tick_params(axis="x", labelsize=14)
    axs.tick_params(axis="y", labelsize=14)
    axs.legend(
        bbox_to_anchor=(1.05, 1),
        loc="upper left",
        fontsize=14,
        title="Component",
        title_fontsize=16,
    )
    axs.set_title(f'Temperature {temp_val} $^o$C')
    plt.xlim([0.0, 1.0])
    plt.ylim([0.0, 0.52])
    plt.gcf().set_size_inches(10, 6)
    plt.show()

