#!/usr/bin/env python
# coding: utf-8

# In[30]:


import matplotlib.pyplot as plt
import numpy as np
from scipy.optimize import fsolve

plt.style.use("ggplot")
plt.rcParams["figure.dpi"] = 400


# In[31]:


n, c_fe2o, c_so4, c_w, x_o2, SM = 50, 2, 2, 55, 0.21, np.array([[-1, 1]])


# In[32]:


pre_fac1, pre_fac2, ea_hom = 21.6, 36.8, 4176
pre_fac3, ea_het = 5.07, 3930
temp_ref = 353.15


# In[33]:


a, b, c = -171.2542, 8391.24, 23.24323
h_fe2, h_fe3, h_so4, h_t = 0.1513, 0.1161, 0.1117, 3.34e-4


# In[34]:


tau1 = 500
tau1_span = np.linspace(0, tau1, n)
temp1 = 100
temp1 += 273.15
p1, mcat1, vl1 = 3.5, 0, 1000


# In[35]:


tau2 = 500
tau2_span = np.linspace(0, tau2, n)
temp2 = 100
temp2 += 273.15
p2, mcat2, vl2 = 3.5, 120, 1000


# In[36]:


c_in = [c_fe2o, 0]
c_guess = [c_fe2o, c_fe2o]
c_guess = [x * 0.5 for x in c_guess]


# In[37]:


def henry(T, *args):
    c_fe2, c_fe3, c_so4, a, b, c, h_fe2, h_fe3, h_so4, h_t = args
    den = np.exp(a + b / T + c * np.log(T))
    term1 = c_fe2 * (h_fe2 + h_t * (T - 298.15))
    term2 = c_fe3 * (h_fe3 + h_t * (T - 298.15))
    term3 = c_so4 * (h_so4 + h_t * (T - 298.15))
    num = 10 ** (term1 + term2 + term3)

    return num / den


# In[38]:


def rhom(c_fe2, *args):
    cO2, temp, pre_fac1, pre_fac2, ea_hom, temp_ref = args
    return (
        (pre_fac1 * c_fe2 ** 2 * cO2)
        / (1 + pre_fac1 / pre_fac2 * c_fe2)
        * np.exp(-ea_hom * (1 / temp - 1 / temp_ref))
    )


# In[39]:


def rhet(c_fe2, *args):
    cO2, temp, k, ea_het, temp_ref = args
    return k * np.exp(-ea_het * (1 / temp - 1 / temp_ref)) * np.sqrt(cO2) * c_fe2


# In[40]:


def ObjFun(c_out, *args):
    (c_in, c_so4, tau, mcat, vl, temp, press, c_w, x_o2, SM, pre_fac1, pre_fac2,
     ea_hom, temp_ref, pre_fac3, ea_het, a, b, c, h_fe2, h_fe3, h_so4, h_t,) = args

    c_fe2, c_fe3 = c_out

    h = henry(temp, c_fe2, c_fe3, c_so4, a, b, c, h_fe2, h_fe3, h_so4, h_t)

    cO2 = x_o2 * press * c_w / h
    rate = (
        rhom(c_fe2, cO2, temp, pre_fac1, pre_fac2, ea_hom, temp_ref)
        + rhet(c_fe2, cO2, temp, pre_fac3, ea_het, temp_ref) * mcat / vl
    )

    result = -c_out + c_in + rate * SM * tau
    return result.reshape(2)


# In[41]:


conc1, conc2 = np.empty([n, 2]), np.empty([n, 2])


# In[42]:


for i in range(n):

    conc1[i, :] = fsolve(
        func=ObjFun,
        x0=c_guess,
        args=(c_in, c_so4, tau1_span[i], mcat1, vl1, temp1, p1, c_w, x_o2, SM, pre_fac1, 
              pre_fac2, ea_hom, temp_ref, pre_fac3, ea_het, a, b, c, h_fe2, h_fe3, h_so4, h_t,),
    )

    conc2[i, :] = fsolve(
        func=ObjFun,
        x0=c_guess,
        args=(conc1[i, :], c_so4, tau2_span[i], mcat2, vl2, temp2, p2, c_w, x_o2, SM, pre_fac1,
              pre_fac2, ea_hom, temp_ref, pre_fac3, ea_het, a, b, c, h_fe2, h_fe3, h_so4, h_t,),
    )


# In[43]:


conv1 = (c_fe2o - conc1[:, 0]) / c_fe2o * 100
conv2 = (conc1[:, 0] - conc2[:, 0]) / conc1[:, 0] * 100
conv_global = (c_fe2o - conc2[:, 0]) / c_fe2o * 100

max_val = np.max(conv_global)

if max_val < 95:
    print("\nRequired conversion not satisfied\n")
    print("Maximun global conversion= %.2f%%\n" % max_val)
else:
    for i in range(n):
        if conv_global[i] > 95:
            print(
                "\nGlobal space-time (95%% global conversion)= %.2f min\n"
                % (tau1_span[i] + tau2_span[i])
            )
            break


# In[44]:


plt.plot(tau1_span, conc1[:, 0], label="Fe$^{2+}$ (Reactor 1)")
plt.plot(tau1_span, conc1[:, 1], label="Fe$^{3+}$ (Reactor 1)")
plt.plot(tau2_span, conc2[:, 0], label="Fe$^{2+}$ (Reactor 2)")
plt.plot(tau2_span, conc2[:, 1], label="Fe$^{3+}$ (Reactor 2)")

plt.title("Concentration vs. Space-time", fontsize=18)
plt.xlabel("Space-time [min]", fontsize=16)
plt.ylabel("Concentration [mol/L]", fontsize=16)
plt.xticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
    title="Concentrations",
    title_fontsize=16,
)
plt.show()


# In[45]:


plt.plot(tau1_span, conv1, label="1$^{st}$ Reactor")
plt.plot(tau2_span, conv2, label="2$^{nd}$ Reactor")
plt.plot(tau1_span + tau2_span, conv_global, label="Global conversion")

plt.title("Conversion vs. Space-time", fontsize=18)
plt.xlabel("Space-time [min]", fontsize=16)
plt.ylabel("Conversion [%]", fontsize=16)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.gcf().set_size_inches(9, 6)
plt.legend(
    bbox_to_anchor=(1.05, 1),
    loc="upper left",
    fontsize=14,
)
plt.show()

